﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Text;

namespace PsiKeepers
{
    public class Background
    {
        private const float BlockSize = 45.0f;

        private int m_w;
        private int m_h;

        private float[,] m_field;
        private float[,] m_field_new;

        public Background(PointF size)
        {
            m_w = (int)(size.X / BlockSize) + 1;
            m_h = (int)(size.Y / BlockSize) + 1;

            m_field = new float[m_h, m_w];
            m_field_new = new float[m_h, m_w];
        }

        public void BeginUpdate()
        {
            for (int y = 0; y < m_h; ++y)
            {
                for (int x = 0; x < m_w; ++x)
                {
                    m_field_new[y, x] = 0.0f;
                }
            }
        }

        public void AddChar(PointF pt)
        {
            int cx = (int)((pt.X + 0.5f) / BlockSize);
            int cy = (int)((pt.Y + 0.5f) / BlockSize);
            if (cx < 0 || cx >= m_w || cy < 0 || cy >= m_h)
                return;

            m_field_new[cy, cx] += 0.2f;
        }

        public void EndUpdate()
        {
            for (int y = 0; y < m_h; ++y)
            {
                for (int x = 0; x < m_w; ++x)
                {
                    // Get the values from surrounding cells
                    float val = 0.0f;
                    for (int cy = -1; cy <= 1; ++cy)
                    {
                        for (int cx = -1; cx <= 1; ++cx)
                        {
                            float amt = 1.0f - (float)(Math.Abs(cx) + Math.Abs(cy)) * 0.3f;
                            val += GetNew(x + cx, y + cy) * amt;
                        }
                    }

                    // Change the value
                    m_field[y, x] = MathUtil.Lerp(m_field[y, x], val, 0.05f);
                }
            }
        }

        public void Draw(Graphics g)
        {
            SmoothingMode orig_smooth = g.SmoothingMode;
            g.SmoothingMode = SmoothingMode.None;

            for (int cy = 0; cy < m_h; ++cy)
            {
                for (int cx = 0; cx < m_w; ++cx)
                {
                    //float val = MathUtil.Clamp(m_field[cy, cx], 0.0f, 1.0f);
                    float val = m_field[cy, cx];
                    int cr = MathUtil.Clamp((int)(val * 200.0f), 0, 255);
                    int cg = MathUtil.Clamp((int)(val * 230.0f), 0, 255);
                    int cb = MathUtil.Clamp((int)(val * 120.0f), 0, 255);
                    Brush brush = new SolidBrush(Color.FromArgb(cr, cg, cb));
                    g.FillRectangle(brush, cx * BlockSize, cy * BlockSize, BlockSize, BlockSize);
                }
            }

            g.SmoothingMode = orig_smooth;
        }

        private float GetNew(int cx, int cy)
        {
            if (cx < 0 || cx >= m_w || cy < 0 || cy >= m_h)
                return 0.0f;
            return m_field_new[cy, cx];
        }
    }
}
