﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;

namespace PsiKeepers
{
    public static class PointFHelper
    {
        public static PointF Add(this PointF p1, PointF p2)
        {
            return new PointF(p1.X + p2.X, p1.Y + p2.Y);
        }

        public static PointF AddMul(this PointF p1, PointF p2, float scale)
        {
            return new PointF(p1.X + p2.X * scale, p1.Y + p2.Y * scale);
        }

        public static PointF Sub(this PointF p1, PointF p2)
        {
            return new PointF(p1.X - p2.X, p1.Y - p2.Y);
        }

        public static PointF Mul(this PointF pt, float scale)
        {
            return new PointF(pt.X * scale, pt.Y * scale);
        }

        public static float Dot(this PointF p1, PointF p2)
        {
            return p1.X * p2.X + p1.Y * p2.Y;
        }

        public static float Norm(this PointF p1)
        {
            return Dot(p1, p1);
        }

        public static float Abs(this PointF p1)
        {
            return (float)Math.Sqrt(Norm(p1));
        }

        public static float DistanceTo(this PointF p1, PointF p2)
        {
            return Sub(p2, p1).Abs();
        }

        public static PointF Unit(this PointF p1)
        {
            float len = Abs(p1);
            if (len < 0.0001f)
                throw new Exception("Trying to normalize a zero vector!");
            return Mul(p1, 1.0f / len);
        }

        public static PointF UnitAndDist(this PointF p1, out float dist)
        {
            float len = Abs(p1);
            if (len < 0.0001f)
                throw new Exception("Trying to normalize a zero vector!");
            dist = len;
            return Mul(p1, 1.0f / len);
        }

        public static PointF UnitOrZero(this PointF p1)
        {
            float len = Abs(p1);
            if (len < 0.0001f)
                return PointF.Empty;
            return Mul(p1, 1.0f / len);
        }

        public static PointF UnitOrZeroAndDist(this PointF p1, out float dist)
        {
            float len = Abs(p1);
            if (len < 0.0001f)
            {
                dist = 0.0f;
                return PointF.Empty;
            }
            dist = len;
            return Mul(p1, 1.0f / len);
        }

        public static PointF AngleToDir(float rad)
        {
            return new PointF((float)Math.Cos(rad), (float)Math.Sin(rad));
        }

        public static float ToAngle(this PointF p1)
        {
            return (float)Math.Atan2(p1.Y, p1.X);
        }

        public static PointF ClampToRect(this PointF p, PointF p1, PointF p2)
        {
            return new PointF(MathUtil.Clamp(p.X, p1.X, p2.X), MathUtil.Clamp(p.Y, p1.Y, p2.Y));
        }
    }
}
